@@ -53,6 +53,7 @@ from chessengine.utils import (
     change_turn,
 )
 from chessengine.pgn.parser import PGNParser, SAN_MOVE_REGEX
+from chessengine.pgn.utils import best_move_from_tree
 
 
 class Board:
@@ -82,41 +83,7 @@ class Board:
         self.black_queens = 576460752303423488  # (D8)
         self.black_kings = 1152921504606846976  # (E8)
 
-        self.all_white = (
-            self.white_pawns
-            | self.white_rooks
-            | self.white_knights
-            | self.white_bishops
-            | self.white_queens
-            | self.white_kings
-        )
-
-        self.all_black = (
-            self.black_pawns
-            | self.black_rooks
-            | self.black_knights
-            | self.black_bishops
-            | self.black_queens
-            | self.black_kings
-        )
-
-        self.all_pieces = self.all_black | self.all_white
-
         self.score = 0
-        self.piece_count = {
-            ("white", "kings"): 1,
-            ("white", "queens"): 1,
-            ("white", "rooks"): 2,
-            ("white", "bishops"): 2,
-            ("white", "knights"): 2,
-            ("white", "pawns"): 8,
-            ("black", "kings"): 1,
-            ("black", "queens"): 1,
-            ("black", "rooks"): 2,
-            ("black", "bishops"): 2,
-            ("black", "knights"): 2,
-            ("black", "pawns"): 8,
-        }
 
         if side.lower().strip() not in ["black", "white"]:
             raise ValueError(f'side must be one of "black" or "white". Got {side}')
@@ -128,9 +95,16 @@ class Board:
         self.black_king_side_castle = True
         self.black_queen_side_castle = True
 
-        # A dictionary matching a side and piece to its corresponding bit board.
-        # Useful when we want to iterate through all of the bitboards of the board.
-        self.board = {
+        # Keep track of all moves made
+        self.moves = []
+
+    @property
+    def board(self):
+        """
+        A dictionary mapping a side and piece to its corresponding bitboard.
+        Useful when we want to iterate over all the bitboards of the board
+        """
+        return {
             ("white", "kings"): self.white_kings,
             ("white", "queens"): self.white_queens,
             ("white", "rooks"): self.white_rooks,
@@ -145,8 +119,44 @@ class Board:
             ("black", "pawns"): self.black_pawns,
         }
 
-        # Keep track of all moves made
-        self.moves = []
+    @property
+    def all_white(self):
+        return (
+            self.white_pawns
+            | self.white_rooks
+            | self.white_knights
+            | self.white_bishops
+            | self.white_queens
+            | self.white_kings
+        )
+
+    @property
+    def all_black(self):
+        return (
+            self.black_pawns
+            | self.black_rooks
+            | self.black_knights
+            | self.black_bishops
+            | self.black_queens
+            | self.black_kings
+        )
+
+    @property
+    def all_pieces(self):
+        return (
+            self.black_pawns
+            | self.black_rooks
+            | self.black_knights
+            | self.black_bishops
+            | self.black_queens
+            | self.black_kings
+            | self.white_pawns
+            | self.white_rooks
+            | self.white_knights
+            | self.white_bishops
+            | self.white_queens
+            | self.white_kings
+        )
 
     def __repr__(self):
         piece_list = ["." for _ in range(64)]
@@ -344,47 +354,6 @@ class Board:
         """
         return self.get_bitboard(side=self.side, piece=piece)
 
-    def update_board_state(self) -> None:
-        """
-        Updates all Board attributes when a move is made or the state of the Board
-        changes in any way. You should call this if you manually make any changes to
-        bitboards attributes, otherwise it is called automatically.
-        """
-        self.all_white = (
-            self.white_pawns
-            | self.white_rooks
-            | self.white_knights
-            | self.white_bishops
-            | self.white_queens
-            | self.white_kings
-        )
-
-        self.all_black = (
-            self.black_pawns
-            | self.black_rooks
-            | self.black_knights
-            | self.black_bishops
-            | self.black_queens
-            | self.black_kings
-        )
-
-        self.all_pieces = self.all_black | self.all_white
-
-        self.board = {
-            ("white", "kings"): self.white_kings,
-            ("white", "queens"): self.white_queens,
-            ("white", "rooks"): self.white_rooks,
-            ("white", "bishops"): self.white_bishops,
-            ("white", "knights"): self.white_knights,
-            ("white", "pawns"): self.white_pawns,
-            ("black", "kings"): self.black_kings,
-            ("black", "queens"): self.black_queens,
-            ("black", "rooks"): self.black_rooks,
-            ("black", "bishops"): self.black_bishops,
-            ("black", "knights"): self.black_knights,
-            ("black", "pawns"): self.black_pawns,
-        }
-
     def set_bitboard(self, side: str, piece: str, board: int) -> None:
         """
         Sets the bitboard for the passed arguments to the passed bitboard
@@ -395,7 +364,6 @@ class Board:
         """
         attrname = side + "_" + piece
         setattr(self, attrname, board)
-        self.update_board_state()
 
     def identify_piece_at(self, position: int) -> tuple:
         """
@@ -514,7 +482,6 @@ class Board:
                     black_pawn_bb = self.get_bitboard("black", "pawns")
                     black_pawn_bb &= clear_position[end >> 8]
                     self.set_bitboard("black", "pawns", black_pawn_bb)
-                    self.piece_count[("black", "pawns")] -= 1
                     self.en_passant_position = 0
 
                 # Clear self.en_passant_position
@@ -532,7 +499,6 @@ class Board:
                     white_pawn_bb = self.get_bitboard("white", "pawns")
                     white_pawn_bb &= clear_position[end << 8]
                     self.set_bitboard("white", "pawns", white_pawn_bb)
-                    self.piece_count[("white", "pawns")] -= 1
                     self.en_passant_position = 0
 
                 # Clear self.en_passant_position
@@ -547,7 +513,6 @@ class Board:
             opp_side_board = self.get_bitboard(end_side, end_piece)
             opp_side_board &= clear_position[end]
             self.set_bitboard(end_side, end_piece, opp_side_board)
-            self.piece_count[(end_side, end_piece)] -= 1
 
         # Clear the moved piece's original position (set "start" to 0)
         move_side_board = self.get_bitboard(start_side, start_piece)
@@ -616,7 +581,7 @@ class Board:
         :raises MoveError: If an ambiguous or invalid move was passed
         :raises PGNParsingError: If the move was invalid SAN
         """
-        if "0-0-0" in move:
+        if "0-0-0" in move or "O-O-O" in move:
             # queen side castle
             if side == "white":
                 if self.white_queen_side_castle:
@@ -636,7 +601,7 @@ class Board:
                     )
                 self.black_queen_side_castle = False
                 self.black_king_side_castle = False
-        elif "0-0" in move:
+        elif "0-0" in move or "O-O" in move:
             # king side castle
             if side == "white":
                 if self.white_king_side_castle:
@@ -668,13 +633,50 @@ class Board:
             else:
                 piece_moved = san_piece_map[groups[0].upper()]
 
-            end_pos = 2 ** coords_to_pos[groups[3].upper()]
+            end_pos = 2 ** coords_to_pos[groups[2].upper()]
             moves = self.get_moves(side, piece_moved)
+
             if groups[1] is None:
-                # No file provided in the SAN
+                # No rank or file provided in the SAN
                 candidate_move = None
                 for m in moves:
                     if m[1] == end_pos:
+                        if candidate_move:
+                            raise MoveError(
+                                f"{move} is ambiguous for {side}. Specify a file or rank to move from."
+                            )
+                        candidate_move = m
+                if candidate_move is None:
+                    raise MoveError(f"{move} is not a valid move for {side}.")
+                self.move(
+                    start=candidate_move[0],
+                    end=candidate_move[1],
+                    score=candidate_move[2],
+                )
+            elif groups[1].isalpha():
+                # File provided in the SAN
+                candidate_move = None
+                for m in moves:
+                    file = get_file(m[0])
+                    if groups[1].upper() == "ABCDEFGH"[file - 1] and m[1] == end_pos:
+                        if candidate_move:
+                            raise MoveError(
+                                f"{move} is ambiguous for {side}. Specify a rank to move from."
+                            )
+                        candidate_move = m
+                if candidate_move is None:
+                    raise MoveError(f"{move} is not valid for {side}.")
+                self.move(
+                    start=candidate_move[0],
+                    end=candidate_move[1],
+                    score=candidate_move[2],
+                )
+            elif groups[1].isnumeric():
+                # Rank provided in the SAN
+                candidate_move = None
+                for m in moves:
+                    rank = get_rank(m[0])
+                    if groups[1] == "12345678"[rank - 1] and m[1] == end_pos:
                         if candidate_move:
                             raise MoveError(
                                 f"{move} is ambiguous for {side}. Specify a file to move from."
@@ -682,33 +684,20 @@ class Board:
                         candidate_move = m
                 if candidate_move is None:
                     raise MoveError(f"{move} is not a valid move for {side}.")
+                self.move(
+                    start=candidate_move[0],
+                    end=candidate_move[1],
+                    score=candidate_move[2],
+                )
+            elif groups[1].isalnum():
+                # Both rank and file provided in the SAN
+                start_pos = 2 ** coords_to_pos[groups[1].upper()]
+                for m in moves:
+                    if m[0] == start_pos and m[1] == end_pos:
+                        self.move(start=start_pos, end=end_pos, score=m[2])
+                        break
                 else:
-                    self.move(start=candidate_move[0], end=candidate_move[1])
-            else:
-                if groups[2] is None:
-                    # No rank provided in the SAN
-                    candidate_move = None
-                    for m in moves:
-                        file = get_file(m[0])
-                        if (
-                            groups[1].upper() == "ABCDEFGH"[file - 1]
-                            and m[1] == end_pos
-                        ):
-                            if candidate_move:
-                                raise MoveError(
-                                    f"{move} is ambiguous for {side}. Specify a file as well as rank to move from."
-                                )
-                            candidate_move = m
-                    if candidate_move is None:
-                        raise MoveError(f"{move} is not a valid move for {side}.")
-                    else:
-                        self.move(start=candidate_move[0], end=candidate_move[1])
-                else:
-                    # File and rank both present in the SAN
-                    start_pos = 2 ** coords_to_pos[groups[1].upper() + groups[2]]
-                    if (start_pos, end_pos) not in moves:
-                        raise MoveError(f"{move} is not a valid move for {side}.")
-                    self.move(start=start_pos, end=end_pos)
+                    raise MoveError(f"{move} is not a valid move for {side}.")
 
     def make_moves(self, *moves: Iterable[tuple[int, int]]) -> None:
         """
@@ -747,11 +736,10 @@ class Board:
             self.move(start=start, end=end, track=False)
             if side is not None:
                 self.set_bitboard(side, piece, board)
-                self.piece_count[(side, piece)] += 1
 
     def get_moves(
         self, side: str, piece: str = None, position: int = None
-    ) -> list[tuple[int, int]]:
+    ) -> list[tuple[int, int, int]]:
         """
         Get all end positions a piece of side can reach starting from position.
         ``side`` is always required, piece and position are optional.
@@ -806,7 +794,7 @@ class Board:
                     moves.extend(self.get_moves(side, piece, position))
             return moves
 
-    def search_forward(self, depth: int = 4) -> tuple[int, tuple[int, int]]:
+    def search_forward(self, depth: int = 4) -> tuple[int, tuple[int, int, int]]:
         """
         Execute an alpha-beta pruned depth-first search to find the optimal move from
         the current board state.
@@ -972,12 +960,12 @@ class Board:
         side_to_move = "white"
         in_game_tree = parser is not None
         current_node = parser.root_node if in_game_tree else None
-        lines_printed = 20
+        lines_printed = 11
         last_move = ""
         while True:
             clear_lines(lines_printed)
             print(self)
-            lines_printed = 20
+            lines_printed = 11
             if side_to_move == self.side:
                 if in_game_tree:
                     move, node = random.choice(list(current_node.children.items()))
